/*
 * ProFTPD - FTP server daemon
 * Copyright (c) 1997, 1998 Public Flood Software
 * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
 * Copyright (c) 2001-2007 The ProFTPD Project team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 *
 * As a special exemption, Public Flood Software/MacGyver aka Habeeb J. Dihu
 * and other respective copyright holders give permission to link this program
 * with OpenSSL, and distribute the resulting executable, without including
 * the source code for OpenSSL in the source distribution.
 */

/*
 * Set sticky bit on direcoty when make directory (mkdir)
 * Copyright (c) JoungKyun.Kim <http://oops.org>
 * $Id: mod_sticky.c,v 1.1.1.1 2008/02/04 17:57:47 oops Exp $
 */

#include "conf.h"
#include "privs.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#define MOD_STICKY_VERSION		"mod_sticky/0.1"

module sticky_module;
short sticky_debug_flag = 0;

void pr_sticky_log (const char *fmt, ...) {
	char buf[PR_TUNABLE_BUFFER_SIZE] = { 0, };
	va_list msg;

	if ( ! sticky_debug_flag )
		return;

	va_start (msg, fmt);
	vsnprintf (buf, sizeof (buf), fmt, msg);
	va_end (msg);

	pr_log_debug (DEBUG0, "%s: %s", MOD_STICKY_VERSION, (const char *) buf);
}

/* Declaration of command handler */
MODRET set_sticky (cmd_rec *cmd) {
	int bool = -1;
	config_rec *c = NULL;

	CHECK_ARGS (cmd, 1);
	CHECK_CONF (cmd, CONF_VIRTUAL|CONF_ROOT|CONF_GLOBAL|CONF_ANON|CONF_DIR|CONF_DYNDIR);

	bool = get_boolean (cmd, 1);
	if ( bool == -1 )
		CONF_ERROR (cmd, _("expected Boolean parameter"));

	c = add_config_param(cmd->argv[0], 1, NULL);
	c->argv[0] = pcalloc(c->pool, sizeof(unsigned char));
	*((unsigned char *) c->argv[0]) = bool;

	return PR_HANDLED (cmd);
}

MODRET set_sticky_debug (cmd_rec *cmd) {
	int bool = -1;
	config_rec *c = NULL;

	CHECK_ARGS (cmd, 1);
	CHECK_CONF (cmd, CONF_VIRTUAL|CONF_ROOT|CONF_GLOBAL|CONF_ANON|CONF_DIR|CONF_DYNDIR);

	bool = get_boolean (cmd, 1);
	if ( bool == -1 )
		CONF_ERROR (cmd, _("expected Boolean parameter"));

	c = add_config_param(cmd->argv[0], 1, NULL);
	c->argv[0] = pcalloc(c->pool, sizeof(unsigned char));
	*((unsigned char *) c->argv[0]) = bool;

	return PR_HANDLED (cmd);
}

MODRET post_cmd_sticky (cmd_rec *cmd) {
	config_rec * sticky_v, * sticky_d;
	struct stat s;

	/*
	 * sticky_v value is
	 *     NULL   => Directive not set
	 *     TRUE   => Use
	 *     FALSE  => Don't use
	 */
	sticky_v = get_param_ptr (CURRENT_CONF, "UseSticky", FALSE);

	/* get Debug flag */
	sticky_d = get_param_ptr (CURRENT_CONF, "UseStickyDebug", FALSE);

	if ( sticky_d && * (unsigned char *) sticky_d == TRUE ) {
		sticky_debug_flag = 1;

		if ( sticky_v )
			pr_sticky_log ("UseSticky set %s", (* (unsigned char *) sticky_v != TRUE) ? "Off" : "On");
		else
			pr_sticky_log ("UseSticky set %s", "NULL");
	}

	/*
	 * if UseSticky not set or set off, skip this function.
	 */
	if ( ! sticky_v || sticky_v == FALSE )
		return 0;

	pr_sticky_log ("post_cmd_sticky arg %d => %s", 0, cmd->argv[0]);
	pr_sticky_log ("post_cmd_sticky arg %d => %s", 1, cmd->argv[1]);

	if ( strcmp (cmd->argv[0], "MKD") )
		return 0;

	if ( stat (cmd->argv[1], &s) < 0 ) {
		pr_sticky_log ("%s is not found", cmd->argv[1]);
		return 0;
	}

	if ( ! S_ISDIR (s.st_mode) ) {
		pr_sticky_log ("%s is not directory", cmd->argv[1]);
		return 0;
	}

	if ( chmod (cmd->argv[1], s.st_mode|S_ISVTX) != 0 ) {
		pr_log_debug (DEBUG0, "%s: add sticky set failed (%s)",
				MOD_STICKY_VERSION, (const char *) strerror (errno));
		return 0;
	}

	return PR_DECLINED (cmd);
}

static conftable sticky_conftab[] = {
	{ "UseSticky",      set_sticky,       NULL },
	{ "UseStickyDebug", set_sticky_debug, NULL },
	{ 0,                NULL,             NULL }
};

static cmdtable sticky_cmdtab[] = {
	{ POST_CMD, C_MKD, G_NONE, post_cmd_sticky, FALSE, FALSE },
	{ 0, NULL }
};

/* Module interface */

module sticky_module = {
	NULL, NULL,	            /* Always NULL */
	0x20,                   /* Module API version (2.0) */
	"sticky",               /* Module name */
	sticky_conftab,         /* Module configuration directive table */
	sticky_cmdtab,          /* Module command handler table */
	NULL,                   /* Module authentication handler table */
	NULL,                   /* Module initialization function */
	NULL,                   /* Session initialization function */
	MOD_STICKY_VERSION    /* Module version */
};
