/*
 * ProFTPD - FTP server daemon
 * Copyright (c) 1997, 1998 Public Flood Software
 * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
 * Copyright (c) 2001-2007 The ProFTPD Project team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 *
 * As a special exemption, Public Flood Software/MacGyver aka Habeeb J. Dihu
 * and other respective copyright holders give permission to link this program
 * with OpenSSL, and distribute the resulting executable, without including
 * the source code for OpenSSL in the source distribution.
 */

/*
 * Time Limit module for ProFTPD
 * Copyright (c) JoungKyun.Kim <http://oops.org>
 * $Id: mod_timelimit.c 10 2014-03-14 18:20:30Z oops $
 */

#include "conf.h"
#include "privs.h"
#include "version.h"

typedef struct tm tm_c;

#define MOD_TIMELIMIT_VERSION		"mod_timelimit/0.2"
#define TIMELIMIT_MESSAGE			_("Sorry, out of operation time ( %B:00 ~ %b:00 )")

#if PROFTPD_VERSION_NUMBER < 0x0001030000
#define tm_log_auth log_auth
#define TM_LOG_NOTICE LOG_NOTICE
#else
#define tm_log_auth pr_log_auth
#define TM_LOG_NOTICE PR_LOG_NOTICE
#endif

module timelimit_module;

/* Declaration of command handler */
MODRET set_timelimit (cmd_rec *cmd) {
	int uptime;
	config_rec * c = NULL;

	if ( cmd->argc < 2 || cmd->argc > 3 )
		CONF_ERROR (cmd, _("invalid number of arguments"));

	CHECK_CONF(cmd,CONF_ROOT|CONF_VIRTUAL|CONF_ANON|CONF_GLOBAL);

	if ( ! strcasecmp (cmd->argv[1], "none" ) )
		uptime = 0;
	else {
		char *endp = NULL;

		uptime = (int) strtol (cmd->argv[1], &endp, 10);
		if ( (endp && *endp) || uptime < 0 )
			CONF_ERROR (cmd, _("argument must be 'none' or a number greater than -1."));
	}

	if ( cmd->argc == 3 ) {
		c = add_config_param (cmd->argv[0], 2, NULL, NULL);
		c->argv[0] = pcalloc (c->pool, sizeof (unsigned int));
		*((unsigned int *) c->argv[0]) = uptime;
		c->argv[1] = pstrdup(c->pool,cmd->argv[2]);
	} else {
		c = add_config_param (cmd->argv[0], 1, NULL);
		c->argv[0] = pcalloc (c->pool, sizeof (unsigned int));
		*((unsigned int *) c->argv[0]) = uptime;
	}

	return PR_HANDLED (cmd);
}

MODRET timelimit_check (cmd_rec *cmd) {
	config_rec * up, * dn;
	char up_time[10], dn_time[10], * tl_str;
	int up_t = 0, dn_t = 24;
	int * ut = NULL, * dt = NULL;
	int ct;
	tm_c * tc;
	time_t t = time (0);

	up = find_config (session.anon_config ? session.anon_config->subset :
			cmd->server->conf, CONF_PARAM, "UpTime", FALSE);
	dn = find_config (session.anon_config ? session.anon_config->subset :
			cmd->server->conf, CONF_PARAM, "DownTime", FALSE);

	if ( ! up && ! dn )
		return PR_DECLINED (cmd);

	tl_str = TIMELIMIT_MESSAGE;

	if ( up ) {
		ut = up->argv[0];
		if ( up->argc > 1 ) tl_str = up->argv[1];
		up_t = * ut;
	}

	if ( dn ) {
		dt = dn->argv[0];
		if (dn->argc > 1) tl_str = dn->argv[1];
		dn_t = *dt;
	}

	snprintf (up_time, sizeof (up_time), "%d", up_t);
	snprintf (dn_time, sizeof (dn_time), "%d", dn_t);

	tc = localtime (&t);
	ct = tc->tm_hour;

	if ( dn_t < up_t ) {
		if ( ct < dn_t ) ct += 24;
		dn_t += 24;
	}

	if ( (ct < up_t) || (ct >= dn_t) ) {
		tm_log_auth (TM_LOG_NOTICE, _("%s: Login refused [%s:00 ~ %s:00]"),
				MOD_TIMELIMIT_VERSION, up_time, dn_time);

		pr_response_send (R_230, "%s",
				sreplace (cmd->tmp_pool,
							sreplace (cmd->tmp_pool, tl_str, "%B", up_time, NULL),
							"%b", dn_time, NULL)
		);
		end_login (0);
	}

	if ( up || dn )
		pr_response_add(R_230, "\n"
				"-------------------------------------------------------------------------------\n"
				"%s : %s:00 ~ %s:00\n"
				"-------------------------------------------------------------------------------",
				_("Service available time"), up_time, dn_time);

	return PR_DECLINED (cmd);
}

static conftable timelimit_conftab[] = {
	{ "UpTime",   set_timelimit, NULL },
	{ "DownTime", set_timelimit, NULL },
	{ 0,          NULL,          NULL }
};

static cmdtable timelimit_cmdtab[] = {
	{ POST_CMD, C_PASS, G_NONE, timelimit_check, FALSE, FALSE, CL_AUTH },
	{ 0, NULL }
};

/* Module interface */

module timelimit_module = {
	NULL, NULL,	            /* Always NULL */
	0x20,                   /* Module API version (2.0) */
	"timelimit",            /* Module name */
 	timelimit_conftab,       /* Module configuration directive table */
	timelimit_cmdtab,        /* Module command handler table */
	NULL,                   /* Module authentication handler table */
	NULL,                   /* Module initialization function */
	NULL,                   /* Session initialization function */
	MOD_TIMELIMIT_VERSION    /* Module version */
};
